# -*- coding: utf8 -*-

from __future__ import unicode_literals

import datetime

parser_class = "acl9000"


def test_encode_order(self):
    message = self.order()
    expected = """
    H|\\^&||||||||ACL9000||P|1|20130715141912
    P|1||||NYME^ANO||20110513|F||||||||||||||||||||||||||
    O|1|123456||^^^A|A|||||||||||PRESCRIBER||||||||||O||||||||
    O|2|123456||^^^B|A|||||||||||PRESCRIBER||||||||||O||||||||
    L|1|N"""
    assert message == expected


def test_decode_query(self):
    worklist = self.decode(
        """
    H|\\^&|||ACL9000|||||||P|1|20130709113653
    Q|1|^SAMPLE1||||||||O
    L|1|N"""
    )
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "SAMPLE1"}],
    }
    assert worklist == expected


def test_encode_void_order(self):
    message = self.encode(
        self.void_worklist,
        context={
            "query": self.decode(
                """
        H|\\^&|||ACL9000|||||||P|1|20130709113653
        Q|1|^SAMPLE1||||||||O
        L|1|N"""
            )
        },
    )
    expected = """
    H|\\^&||||||||ACL9000||P|1|20130715141912
    L|1|N
    """
    assert message == expected


def test_decode_result(self):
    worklist = self.decode(
        b"""
    H|\\^&|||ACL9000|||||||P|1|20130709115304
    P|1||||TEST Anonyme|||U|||||||||||||||||DEP||||||||
    O|1|16096131||^^^0501|||||||||||^|||||||||||F||||||
    R|1|^^^0501|49.500|s||||F||||20130709115300|
    C|1|I|300^Resultat hors limites normales|I
    R|2|^^^0501|3.543|R||||F||||20130709115300|
    R|3|^^^0501|4.177|INR||||F||||20130709115300|
    C|1|I|300^Resultat hors limites normales|I
    R|4|^^^0501|16.982|%||||F||||20130709115300|
    C|1|I|300^Resultat hors limites normales|I
    R|5|^^^0229|0.143|\x7f Abs||||F||||20130709115300|
    C|1|I|79^Alerte: Emul.R\x82f presque vide|I
    C|2|I|83^Alerte: Nettoyage non r\x82alis\x82|I
    C|3|I|-1^Alerte: La stabilit\x82 du r\x82actif \x85 bord est depass\x82.|I
    L|1|N""".decode(
            self.parser.ENCODING
        )
    )
    date = datetime.datetime(2013, 7, 9, 11, 53)
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "16096131",
                "type": "patient",
                "results": [
                    {
                        "code": "0501s",
                        "comments": ["Resultat hors limites normales"],
                        "date": date,
                        "value": "49.500",
                    },
                    {"code": "0501R", "comments": [], "date": date, "value": "3.543"},
                    {
                        "code": "0501INR",
                        "comments": ["Resultat hors limites normales"],
                        "date": date,
                        "value": "4.177",
                    },
                    {
                        "code": "0501%",
                        "comments": ["Resultat hors limites normales"],
                        "date": date,
                        "value": "16.982",
                    },
                    {
                        "code": "0229\x7f Abs",
                        "date": date,
                        "value": "0.143",
                        "comments": [
                            "Alerte: Emul.Réf presque vide",
                            "Alerte: Nettoyage non réalisé",
                            "Alerte: La stabilité du réactif à bord est depassé.",
                        ],
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_qc(self):
    worklist = self.decode(
        """
    H|\\^&|||ACL9000|||||||P|1|20131023161535
    P|1||||||||||||||||||||||||||||||||||
    O|1| DDI Bas||^^^0229|||||||Q||||^|||||||||||F||||||
    R|1|^^^0229|640.135|ng/ml||||F||||20131023161516|
    P|2||||||||||||||||||||||||||||||||||
    O|1| DDI Ht||^^^0229|||||||Q||||^|||||||||||F||||||
    R|1|^^^0229|1389.200|ng/ml||||F||||20131023161516|
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "sample_id": "DDI Bas",
                "control_lot": "DDI Bas",
                "results": [
                    {
                        "code": "0229ng/ml",
                        "comments": [],
                        "date": datetime.datetime(2013, 10, 23, 16, 15, 16),
                        "value": "640.135",
                    }
                ],
            },
            {
                "type": "control",
                "sample_id": "DDI Ht",
                "control_lot": "DDI Ht",
                "results": [
                    {
                        "code": "0229ng/ml",
                        "comments": [],
                        "date": datetime.datetime(2013, 10, 23, 16, 15, 16),
                        "value": "1389.200",
                    }
                ],
            },
        ],
    }
    assert worklist == expected


def test_decode_qc_prefix(self):
    self.configure(qc_prefix="CQ")
    worklist = self.decode(
        """
    H|\\^&|||ACL9000|||||||P|1|20131023134707
    P|1|||||||U||||||||||||||||||||||||||
    O|1|CQHM04_008Y13||^^^0501|||||||||||^|||||||||||F||||||
    R|1|^^^0501|19.200|s||||F||||20131023134700|
    R|2|^^^0501|1.778|R||||F||||20131023134700|
    R|3|^^^0501|1.798|INR||||F||||20131023134700|
    R|4|^^^0501|45.068|%||||F||||20131023134700|
    L|1|N"""
    )
    date = datetime.datetime(2013, 10, 23, 13, 47)
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "sample_id": "CQHM04_008Y13",
                "control_lot": "CQHM04_008Y13",
                "results": [
                    {"code": "0501s", "comments": [], "date": date, "value": "19.200"},
                    {"code": "0501R", "comments": [], "date": date, "value": "1.778"},
                    {"code": "0501INR", "comments": [], "date": date, "value": "1.798"},
                    {"code": "0501%", "comments": [], "date": date, "value": "45.068"},
                ],
            }
        ],
    }
    assert worklist == expected
