# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import datetime

import pytest

from driverng import Worklist

parser_class = "elo"


def test_decode_message_query(self):
    worklist = self.decode(
        """
        H|\\^&|||ELO|||||||P||20180731143826
        Q|1|003467891||ALL||||||||O
        L|1|N
        """
    )
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "003467891"}],
    }
    assert worklist == expected


def test_encode_worklist_order(self):
    worklist = Worklist(
        context="order",
        sender_id="SENDER_ID",
        receiver_id="RECEIVER_ID",
        samples=[
            {
                "sample_id": "0102030405",
                "type": "patient",
                "last_name": "Z_LABO",
                "first_name": "Tasty",
                "title": "Mme",
                "sex": "F",
                "patient_id": "0102030405",
                "birthdate": datetime.datetime(1990, 1, 28),
                "request_date": datetime.datetime(2018, 7, 31, 15, 10, 24),
                "specimen": "S",
                "orders": [
                    {"code": "A", "dilution": "100", "action": "add"},
                    {"code": "B", "dilution": "50", "action": "add"},
                ],
            }
        ],
    )
    message = self.encode(
        worklist,
        context={"now": datetime.datetime(2018, 7, 31, 15, 10, 24), "context": "order"},
    )
    expected = """
    H|\\^&|||SENDER_ID|||||||P||20180731151024
    P|1|0102030405|||Z_LABO^Tasty||19900128|F
    O|1|0102030405||^^^A^100\\^^^B^50||20180731151024|||||||||S||||||||||O
    L|1|N
    """
    assert message == expected


def test_decode_message_resultats(self):
    worklist = self.decode(
        b"""
    H|\\^&|||SENDER_ID|||||||P||20180731143826
    P|1|0102030405|||Z_LABO^Tasty||19900128|F
    O|1|0102030405||^^^A^100||20180731151024|||||||||S|||||||||BPX1|F
    R|1|^^^A^100|pos
    O|2|0102030405||^^^B^50||20180731151024|||||||||S|||||||||BPX1|F
    R|1|^^^B^50|Resu With Space
    C|1|Je suis un commentaire avec caracteres sp\xe9ciaux
    O|3|0102030405||^^^C||20180731151024|||||||||S|||||||||BPX2|F
    R|1|^^^C|1050
    O|4|0102030405||^^^R||20180731151024|||||||||S|||||||||BPX2|F
    R|1|^^^R|12,5|Ratio||||S
    C|1|First Comment
    C|2|Second Comment
    O|5|0102030405||^^^U||20180731151024|||||||||S|||||||||BPX3|F
    R|1|^^^U|0,45|mmol/L|0,3-0,8
    L|1|N
    """.decode(
            self.parser.ENCODING
        )
    )
    expected = {
        "context": "result",
        "sender_id": "SENDER_ID",
        "samples": [
            {
                "sample_id": "0102030405",
                "type": "patient",
                "last_name": "Z_LABO",
                "first_name": "Tasty",
                "birthdate": datetime.datetime(1990, 1, 28),
                "sex": "F",
                "specimen": "S",
                "results": [
                    {
                        "code": "A",
                        "dilution": "100",
                        "value": "pos",
                        "unit": "",
                        "instrument": "BPX1",
                        "status": "F",
                    },
                    {
                        "code": "B",
                        "dilution": "50",
                        "value": "Resu With Space",
                        "unit": "",
                        "instrument": "BPX1",
                        "status": "F",
                        "comments": ["Je suis un commentaire avec caracteres spéciaux"],
                    },
                    {
                        "code": "C",
                        "dilution": "",
                        "value": "1050",
                        "unit": "",
                        "instrument": "BPX2",
                        "status": "F",
                    },
                    {
                        "code": "R",
                        "dilution": "",
                        "value": "12,5",
                        "unit": "Ratio",
                        "instrument": "BPX2",
                        "status": "S",
                        "comments": ["First Comment", "Second Comment"],
                    },
                    {
                        "code": "U",
                        "dilution": "",
                        "value": "0,45",
                        "unit": "mmol/L",
                        "instrument": "BPX3",
                        "status": "F",
                    },
                ],
            }
        ],
    }
    assert worklist == expected


@pytest.mark.parametrize(
    "worklist_kwargs",
    [
        {
            "context": "order",
            "sender_id": "SENDER_ID",
            "receiver_id": "RECEIVER_ID",
            "samples": [
                {
                    "sample_id": "",
                    "type": "patient",
                    "last_name": "Z_LABO",
                    "first_name": "Tasty",
                    "title": "Mme",
                    "sex": "F",
                    "patient_id": "0102030405",
                    "birthdate": datetime.datetime(1990, 1, 28),
                    "physicians": [{"code": "55841", "type": "prescriber"}],
                    "request_date": datetime.datetime(2018, 7, 31, 15, 10, 24),
                    "specimen": "S",
                    "orders": [
                        {"code": "A", "dilution": "100", "action": "add"},
                        {"code": "B", "dilution": "50", "action": "add"},
                    ],
                }
            ],
        },
        {
            "context": "order",
            "sender_id": "SENDER_ID",
            "receiver_id": "RECEIVER_ID",
            "samples": [
                {
                    "sample_id": "0102030405",
                    "type": "patient",
                    "last_name": "Z_LABO",
                    "first_name": "Tasty",
                    "title": "Mme",
                    "sex": "F",
                    "patient_id": "0102030405",
                    "birthdate": datetime.datetime(1990, 1, 28),
                    "physicians": [{"code": "55841", "type": "prescriber"}],
                    "request_date": datetime.datetime(2018, 7, 31, 15, 10, 24),
                    "specimen": "S",
                    "orders": [
                        {"code": "", "dilution": "100", "action": "add"},
                        {"code": "B", "dilution": "50", "action": "add"},
                    ],
                }
            ],
        },
    ],
)
def test_encode_worklist_with_errors(self, worklist_kwargs):
    worklist = Worklist(**worklist_kwargs)
    message = self.encode(
        worklist,
        context={"now": datetime.datetime(2018, 7, 31, 15, 10, 24), "context": "order"},
    )
    expected = """
    H|\\^&||||||||||P||20180731151024
    L|1|E
    """
    assert message == expected


def test_encode_worklist_empty_order(self):
    worklist = Worklist(
        context="order", sender_id="SENDER_ID", receiver_id="RECEIVER_ID", samples=[]
    )
    message = self.encode(
        worklist,
        context={"now": datetime.datetime(2018, 7, 31, 15, 10, 24), "context": "order"},
    )
    expected = """
    H|\\^&|||SENDER_ID|||||||P||20180731151024
    P|1
    L|1|I
    """
    assert message == expected


@pytest.mark.parametrize(
    "elo_dilution_default, worklist_kwargs, expected",
    [
        (  # Pas de dilution par défaut
            None,
            {
                "context": "order",
                "sender_id": "SENDER_ID",
                "receiver_id": "RECEIVER_ID",
                "samples": [
                    {
                        "sample_id": "0102030405",
                        "type": "patient",
                        "last_name": "Z_LABO",
                        "first_name": "Tasty",
                        "title": "Mme",
                        "sex": "F",
                        "patient_id": "0102030405",
                        "birthdate": datetime.datetime(1990, 1, 28),
                        "request_date": datetime.datetime(2018, 7, 31, 15, 10, 24),
                        "specimen": "S",
                        "orders": [
                            {"code": "A", "dilution": "100", "action": "add"},
                            {"code": "B", "dilution": "", "action": "add"},
                        ],
                    }
                ],
            },
            """
            H|\\^&|||SENDER_ID|||||||P||20180731151024
            P|1|0102030405|||Z_LABO^Tasty||19900128|F
            O|1|0102030405||^^^A^100\\^^^B||20180731151024|||||||||S||||||||||O
            L|1|N
            """,
        ),
        (  # avec dilution par défaut mais avec dilution null (présence la dilution mais mise a '')
            "55",
            {
                "context": "order",
                "sender_id": "SENDER_ID",
                "receiver_id": "RECEIVER_ID",
                "samples": [
                    {
                        "sample_id": "0102030405",
                        "type": "patient",
                        "last_name": "Z_LABO",
                        "first_name": "Tasty",
                        "title": "Mme",
                        "sex": "F",
                        "patient_id": "0102030405",
                        "birthdate": datetime.datetime(1990, 1, 28),
                        "request_date": datetime.datetime(2018, 7, 31, 15, 10, 24),
                        "specimen": "S",
                        "orders": [
                            {"code": "A", "dilution": "100", "action": "add"},
                            {"code": "B", "dilution": "", "action": "add"},
                        ],
                    }
                ],
            },
            """
            H|\\^&|||SENDER_ID|||||||P||20180731151024
            P|1|0102030405|||Z_LABO^Tasty||19900128|F
            O|1|0102030405||^^^A^100\\^^^B||20180731151024|||||||||S||||||||||O
            L|1|N
            """,
        ),
        (  # avec dilution pas défaut sans dilution renseignée
            "55",
            {
                "context": "order",
                "sender_id": "SENDER_ID",
                "receiver_id": "RECEIVER_ID",
                "samples": [
                    {
                        "sample_id": "0102030405",
                        "type": "patient",
                        "last_name": "Z_LABO",
                        "first_name": "Tasty",
                        "title": "Mme",
                        "sex": "F",
                        "patient_id": "0102030405",
                        "birthdate": datetime.datetime(1990, 1, 28),
                        "request_date": datetime.datetime(2018, 7, 31, 15, 10, 24),
                        "specimen": "S",
                        "orders": [
                            {"code": "A", "dilution": "100", "action": "add"},
                            {"code": "B", "action": "add"},
                        ],
                    }
                ],
            },
            """
            H|\\^&|||SENDER_ID|||||||P||20180731151024
            P|1|0102030405|||Z_LABO^Tasty||19900128|F
            O|1|0102030405||^^^A^100\\^^^B^55||20180731151024|||||||||S||||||||||O
            L|1|N
            """,
        ),
        (  # avec dilution par défaut et dilution renseignée
            "55",
            {
                "context": "order",
                "sender_id": "SENDER_ID",
                "receiver_id": "RECEIVER_ID",
                "samples": [
                    {
                        "sample_id": "0102030405",
                        "type": "patient",
                        "last_name": "Z_LABO",
                        "first_name": "Tasty",
                        "title": "Mme",
                        "sex": "F",
                        "patient_id": "0102030405",
                        "birthdate": datetime.datetime(1990, 1, 28),
                        "request_date": datetime.datetime(2018, 7, 31, 15, 10, 24),
                        "specimen": "S",
                        "orders": [
                            {"code": "A", "dilution": "100", "action": "add"},
                            {"code": "B", "dilution": "50", "action": "add"},
                        ],
                    }
                ],
            },
            """
            H|\\^&|||SENDER_ID|||||||P||20180731151024
            P|1|0102030405|||Z_LABO^Tasty||19900128|F
            O|1|0102030405||^^^A^100\\^^^B^50||20180731151024|||||||||S||||||||||O
            L|1|N
            """,
        ),
    ],
)
def test_encode_worklist_with_default_dilution_option(
    self, elo_dilution_default, worklist_kwargs, expected
):
    self.configure(elo_dilution_default=elo_dilution_default)
    worklist = Worklist(**worklist_kwargs)
    message = self.encode(
        worklist,
        context={"now": datetime.datetime(2018, 7, 31, 15, 10, 24), "context": "order"},
    )
    assert message == expected


@pytest.mark.parametrize(
    "elo_ignore_partial_results, message, expected",
    [
        (
            False,
            b"""
            H|\\^&|||SENDER_ID|||||||P||20180731143826
            P|1|0102030405|||Z_LABO^Tasty||19900128|F
            O|1|0102030405||^^^A^100||20180731151024|||||||||S|55841||||||||BPX1|F
            R|1|^^^A^100|pos
            O|2|0102030405||^^^B^50||20180731151024|||||||||S|||||||||BPX1|F
            R|1|^^^B^50|Resu With Space
            C|1|Je suis un commentaire avec caracteres sp\xe9ciaux
            O|3|0102030405||^^^C||20180731151024|||||||||S|||||||||BPX1|F
            R|1|^^^C|1050
            O|4|0102030405||^^^R||20180731151024|||||||||S|||||||||BPX1|S
            R|1|^^^R|12,5|Ratio||||S
            C|1|First Comment
            C|2|Second Comment
            O|5|0102030405||^^^U||20180731151024|||||||||S|||||||||BPX1|F
            R|1|^^^U|0,45|mmol/L|0,3-0,8
            L|1|N
            """,
            {
                "context": "result",
                "sender_id": "SENDER_ID",
                "samples": [
                    {
                        "sample_id": "0102030405",
                        "type": "patient",
                        "last_name": "Z_LABO",
                        "first_name": "Tasty",
                        "birthdate": datetime.datetime(1990, 1, 28),
                        "sex": "F",
                        "specimen": "S",
                        "results": [
                            {
                                "code": "A",
                                "dilution": "100",
                                "value": "pos",
                                "unit": "",
                                "instrument": "BPX1",
                                "status": "F",
                            },
                            {
                                "code": "B",
                                "dilution": "50",
                                "value": "Resu With Space",
                                "unit": "",
                                "instrument": "BPX1",
                                "status": "F",
                                "comments": [
                                    "Je suis un commentaire avec caracteres spéciaux"
                                ],
                            },
                            {
                                "code": "C",
                                "dilution": "",
                                "value": "1050",
                                "unit": "",
                                "instrument": "BPX1",
                                "status": "F",
                            },
                            {
                                "code": "R",
                                "dilution": "",
                                "value": "12,5",
                                "unit": "Ratio",
                                "instrument": "BPX1",
                                "status": "S",
                                "comments": ["First Comment", "Second Comment"],
                            },
                            {
                                "code": "U",
                                "dilution": "",
                                "value": "0,45",
                                "unit": "mmol/L",
                                "instrument": "BPX1",
                                "status": "F",
                            },
                        ],
                    }
                ],
            },
        ),
        (
            True,
            b"""
            H|\\^&|||SENDER_ID|||||||P||20180731143826
            P|1|0102030405|||Z_LABO^Tasty||19900128|F
            O|1|0102030405||^^^A^100||20180731151024|||||||||S||||||||||F
            R|1|^^^A^100|pos
            O|2|0102030405||^^^B^50||20180731151024|||||||||S||||||||||F
            R|1|^^^B^50|Resu With Space
            C|1|Je suis un commentaire avec caracteres sp\xe9ciaux
            O|3|0102030405||^^^C||20180731151024|||||||||S||||||||||F
            R|1|^^^C|1050
            O|4|0102030405||^^^R||20180731151024|||||||||S||||||||||S
            R|1|^^^R|12,5|Ratio||||S
            C|1|First Comment
            C|2|Second Comment
            O|5|0102030405||^^^U||20180731151024|||||||||S||||||||||F
            R|1|^^^U|0,45|mmol/L|0,3-0,8
            L|1|N
            """,
            {
                "context": "result",
                "sender_id": "SENDER_ID",
                "samples": [
                    {
                        "sample_id": "0102030405",
                        "type": "patient",
                        "last_name": "Z_LABO",
                        "first_name": "Tasty",
                        "birthdate": datetime.datetime(1990, 1, 28),
                        "sex": "F",
                        "specimen": "S",
                        "results": [
                            {
                                "code": "A",
                                "dilution": "100",
                                "value": "pos",
                                "unit": "",
                                "instrument": "",
                                "status": "F",
                            },
                            {
                                "code": "B",
                                "dilution": "50",
                                "value": "Resu With Space",
                                "unit": "",
                                "instrument": "",
                                "status": "F",
                                "comments": [
                                    "Je suis un commentaire avec caracteres spéciaux"
                                ],
                            },
                            {
                                "code": "C",
                                "dilution": "",
                                "value": "1050",
                                "unit": "",
                                "instrument": "",
                                "status": "F",
                            },
                            {
                                "code": "U",
                                "dilution": "",
                                "value": "0,45",
                                "unit": "mmol/L",
                                "instrument": "",
                                "status": "F",
                            },
                        ],
                    }
                ],
            },
        ),
    ],
)
def test_decode_message_with_ignore_partial_results(
    self, elo_ignore_partial_results, message, expected
):
    self.configure(elo_ignore_partial_results=elo_ignore_partial_results)
    worklist = self.decode(message.decode(self.parser.ENCODING))
    assert worklist == expected


def test_encode_worklist_with_wrong_unknown_sex(self):
    worklist = Worklist(
        context="order",
        sender_id="SENDER_ID",
        receiver_id="RECEIVER_ID",
        samples=[
            {
                "sample_id": "0102030405",
                "type": "patient",
                "last_name": "Z_LABO",
                "first_name": "Tasty",
                "title": "Mme",
                "sex": "U",
                "patient_id": "0102030405",
                "birthdate": datetime.datetime(1990, 1, 28),
                "request_date": datetime.datetime(2018, 7, 31, 15, 10, 24),
                "specimen": "S",
                "orders": [
                    {"code": "A", "dilution": "100", "action": "add"},
                    {"code": "B", "dilution": "50", "action": "add"},
                ],
            }
        ],
    )
    message = self.encode(
        worklist,
        context={"now": datetime.datetime(2018, 7, 31, 15, 10, 24), "context": "order"},
    )
    expected = """
    H|\\^&|||SENDER_ID|||||||P||20180731151024
    P|1|0102030405|||Z_LABO^Tasty||19900128
    O|1|0102030405||^^^A^100\\^^^B^50||20180731151024|||||||||S||||||||||O
    L|1|N
    """
    assert message == expected


def test_decode_message_resultat_qc(self):
    message = b"""
    H|\\^&|||EUROLabOffice4|||||||P||20181115102735
    P|1
    O|1|HM^Level 1^BPX1|50423|^^^HM||20181115102735||||||||||||||||||BPX1|Q
    R|1|^^^HMQUANT|0.1|AI||||F|||2018111510273520181115102735
    R|2|^^^HMQUAL|Negative|||||F|||20181115102735|20181115102735
    P|1
    O|1|VCAM^Level 1^BPX1|50423|^^^VCAM||20181115102735||||||||||||||||||BPX1|Q
    R|1|^^^VCAMQUANT|0.1|AI||||F|||2018111510273520181115102735
    R|2|^^^VCAMQUAL|Negative|||||F|||20181115102735|20181115102735
    L|1|N
    """
    worklist = self.decode(message.decode(self.parser.ENCODING))
    expected = {
        "context": "result",
        "samples": [
            {
                "control_lot": "50423",
                "results": [
                    {"code": "HMQUANT", "instrument": "BPX1", "value": "0.1"},
                    {"code": "HMQUAL", "instrument": "BPX1", "value": "Negative"},
                    {"code": "VCAMQUANT", "instrument": "BPX1", "value": "0.1"},
                    {"code": "VCAMQUAL", "instrument": "BPX1", "value": "Negative"},
                ],
                "sample_id": "50423",
                "type": "control",
            }
        ],
        "sender_id": "EUROLabOffice4",
    }
    assert worklist == expected
