# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

from driverng.worklist import Worklist

parser_class = "remisol_v2"


def test_order(self):
    worklist = Worklist(
        context="order",
        samples=[
            {
                "sample_id": "123456",
                "type": "patient",
                "last_name": "COVER",
                "first_name": "Harry",
                "maiden_name": "POTTER",
                "sex": "M",
                "patient_id": "001",
                "physicians": [
                    {
                        "code": "PRESCRIBER",
                        "last_name": "Dr Jones",
                        "type": "prescriber",
                        "phone": "0123;456",
                    }
                ],
                "birthdate": datetime.datetime(1963, 6, 12),
                "location": "Recovery",
                "specimen": "S",
                "orders": [
                    {"code": "NA", "priority": "R", "action": "add"},
                    {"code": "K", "priority": "R", "action": "add"},
                    {"code": "CL", "priority": "R", "action": "add"},
                ],
            }
        ],
    )
    worklist.samples[0].results = [
        {
            "anterior": True,
            "code": "NA",
            "value": "134.1",
            "date": datetime.datetime(2001, 3, 12, 12, 45),
            "unit": "mmol/L",
        },
        {"anterior": False, "code": "K", "value": "10", "unit": "mmol/L"},
    ]
    message = self.order(worklist=worklist)
    expected = """
    H|\\^&|||MCA^LIS|||||Remisol||P|1|20130715141912
    P|1||001||COVER^Harry|POTTER|19630612|M|||||Dr Jones||||||||||||Recovery
    O|1|123456||^^^NA\\^^^K\\^^^CL|R||||||A||||S
    R|1|^^^NA|134.1|mmol/L||||R||||20010312124500
    R|2|^^^K|10|mmol/L||||R
    L|1
    """
    assert message == expected


def test_numerical_results(self):
    worklist = self.decode(
        """
    H|\\^&|||REMISOL|||||UPLOAD_SIGN=1\\INTERPRETATION_FLAG=0\\INSTRUMENT_FLAG=1||E1394-97|19970815080000
    P|1||001||COVER^Harry||19630612|M|||||Dr Jones||||||||||||Recovery|||||
    O|1|123456^1^1|||R||||||||||S|||||||
    R|1|^^^NA^1|142.3|mmol/L||N||R||||20030127142832|0
    R|2|^^^K^1|3.40|mmol/L||<||R||||20030127142832|1
    R|3|^^^CL^1|121.0|mmol/L||N||R||||20030127142832|1
    L|1
    """
    )
    date = datetime.datetime(2003, 1, 27, 14, 28, 32)
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "NA",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "",
                        "value": "142.3",
                        "unit": "mmol/L",
                    },
                    {
                        "code": "K",
                        "comments": [],
                        "date": date,
                        "flags": ["<"],
                        "instrument": "1",
                        "operator": "",
                        "value": "3.40",
                        "unit": "mmol/L",
                    },
                    {
                        "code": "CL",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "1",
                        "operator": "",
                        "value": "121.0",
                        "unit": "mmol/L",
                    },
                ],
                "sample_id": "123456",
                "rack": "1_1",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_results_interpretations(self):
    worklist = self.decode(
        """
    H|\\^&|||REMISOL|||||UPLOAD_SIGN=1\\INTERPRETATION_FLAG=1\\INSTRUMENT_FLAG=1||E1394-97|19970815080000
    P|1||001||COVER^Harry||19630612|M|||||Dr Jones||||||||||||Recovery|||||
    O|1|123456^1^1|||R||||||||||S|||||||
    R|1|^^^NA^1|142.3^INT POS|mmol/L||N||R||||20030127142832|0
    L|1
    """
    )
    date = datetime.datetime(2003, 1, 27, 14, 28, 32)
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "NA",
                        "comments": [],
                        "date": date,
                        "flags": ["INT", "POS", "N"],
                        "instrument": "0",
                        "operator": "",
                        "value": "142.3",
                        "unit": "mmol/L",
                    }
                ],
                "sample_id": "123456",
                "rack": "1_1",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_results_with_comments(self):
    worklist = self.decode(
        """
    H|\\^&|||REMISOL|||||UPLOAD_SIGN=1\\INTERPRETATION_FLAG=0\\INSTRUMENT_FLAG=0||P|1|20010530152830
    P|1||64343||FIEHL^JACK||19480105|M
    O|1|SE1546515^12^1|||R||200105221525||||||||S
    C|1||this is a technical comment
    C|2||this is the interpretation
    R|1|^^^CKMB^^^^^0|9.0|U/L||N||||||20030227142832|0
    C|3|R|comment1;comment2
    R|2|^^^Cortisol^^^^^0|788.0 (R Z)|ug/dL||A||||||20030227142832|0
    C|4|M|comment3
    R|3|^^^Ferritin^^^^^0|NR|ng/mL||A||||||20030227142832|0
    C|4||comment4
    L|1
    """
    )
    message_date = datetime.datetime(2001, 5, 30, 15, 28, 30)
    date = datetime.datetime(2003, 2, 27, 14, 28, 32)
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "CKMB",
                        "comments": [],
                        "date": date,
                        "flags": ["N", "comment1", "comment2"],
                        "instrument": "0",
                        "operator": "",
                        "value": "9.0",
                        "unit": "U/L",
                    },
                    {
                        "code": "Cortisol",
                        "comments": ["comment3"],
                        "date": date,
                        "flags": ["R", "Z", "A"],
                        "instrument": "0",
                        "operator": "",
                        "value": "788.0",
                        "unit": "ug/dL",
                    },
                    {
                        "code": "Ferritin",
                        "comments": ["comment4"],
                        "date": date,
                        "flags": ["A"],
                        "instrument": "0",
                        "operator": "",
                        "value": "NR",
                        "unit": "ng/mL",
                    },
                    {
                        "code": "COM1",
                        "date": message_date,
                        "value": "this is a technical comment",
                    },
                    {
                        "code": "COM2",
                        "date": message_date,
                        "value": "this is the interpretation",
                    },
                ],
                "sample_id": "SE1546515",
                "rack": "1_12",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_hematology_results(self):
    worklist = self.decode(
        """
    H|\\^&|||REMISOL|||||UPLOAD_SIGN=0\\INTERPRETATION_FLAG=0\\INSTRUMENT_FLAG=0||P|1|20040227120136
    P|1||012512||Othero^||19870519|M|||||40||||||||||||39
    O|1|0453^18^7|||R||20040227120103||||||||Blood
    C|1|I|[1] [FROTTIS/] [3] [ANTERIORITE? FROTTIS/]
    C|2|R|ALARM 1 VERIFY DIFF
    R|1|^^^WBC^^^^^0|17.4|10^3/mm3||N||||SYSADM||19980911141945|0
    C|3|R|DIFF : ABNORMAL LY
    R|2|^^^RBC^^^^^0|3.84|10^6/mm3||N||||REMISOL||19980911141945|0
    C|4|I|1+ Anisocytosis
    R|3|^^^HGB^^^^^0|13.2|g/100mL||N||||REMISOL||19980911141945|0
    R|4|^^^HCT^^^^^0|38.2|%||N||||SYSADM||19980911141945|0
    R|5|^^^MCV^^^^^0|99.4|fL||N||||SYSADM||19980911141945|0
    R|6|^^^MCH^^^^^0|34.4|pg/dL||N||||SYSADM||19980911141945|0
    R|7|^^^MCHC^^^^^0|34.6|g/dL||N||||SYSADM||19980911141945|0
    R|8|^^^RDW^^^^^0|17.0|||N||||SYSADM||19980911141945|0
    C|5|R|MODERATED ANISOCYTOSE
    R|9|^^^PLT^^^^^0|141.0|10^3/mm3||N||||SYSADM||19980911141945|0
    R|10|^^^MPV^^^^^0|8.6|fL||N||||SYSADM||19980911141945|0
    R|11|^^^PCT^^^^^0|0.1210|||N||||SYSADM||19980911141945|0
    R|12|^^^PDW^^^^^0|15.50|||N||||SYSADM||19980911141945|0
    R|13|^^^RET#^^^^^0|0.0|10^3/mm3||N||||SYSADM||19980911141945|0
    R|14|^^^RET%^^^^^0|1.26|%||N||||SYSADM||19980911141945|0
    R|15|^^^NE#^^^^^0|9.03|10^3/mm3||N||||SYSADM||19980911141945|0
    R|16|^^^LY#^^^^^0|7.5|10^3/mm3||N||||SYSADM||19980911141945|0
    R|17|^^^MO#^^^^^0|0.82|10^3/mm3||N||||SYSADM||19980911141945|0
    R|18|^^^BA#^^^^^0|0.03|10^3/mm3||N||||SYSADM||19980911141945|0
    R|19|^^^EO#^^^^^0|0.02|10^3/mm3||N||||SYSADM||19980911141945|0
    R|20|^^^NE%^^^^^0|51.9|%||N||||SYSADM||19980911141945|0
    R|21|^^^LY%^^^^^0|43.1|%||N||||SYSADM||19980911141945|0
    C|6|I|Lymphocytosis #
    C|7|I|Ly Blasts
    R|22|^^^MO%^^^^^0|4.7|%||N||||SYSADM||19980911141945|0
    R|23|^^^BA%^^^^^0|0.2|%||N||||SYSADM||19980911141945|0
    R|24|^^^EO%^^^^^0|0.1|%||N||||SYSADM||19980911141945|0
    R|25|^^^IMR^^^^^0|0.23|||N||||SYSADM||19980911141945|0
    R|26|^^^VMR^^^^^0|115.5|||N||||SYSADM||19980911141945|0
    R|27|^^^VMCS^^^^^0|92.6|||N||||SYSADM||19980911141945|0
    R|28|^^^RFD%^^^^^0|0.3|%||N||||SYSADM||19980911141945|0
    R|29|^^^RFD#^^^^^0|0.0|||N||||SYSADM||19980911141945|0
    L|1
    """
    )
    message_date = datetime.datetime(2004, 2, 27, 12, 1, 36)
    date = datetime.datetime(1998, 9, 11, 14, 19, 45)
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "WBC",
                        "comments": [],
                        "date": date,
                        "flags": ["N", "DIFF : ABNORMAL LY"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "17.4",
                        "unit": "10^3/mm3",
                    },
                    {
                        "code": "RBC",
                        "comments": [],
                        "date": date,
                        "flags": ["N", "1+ Anisocytosis"],
                        "instrument": "0",
                        "operator": "REMISOL",
                        "value": "3.84",
                        "unit": "10^6/mm3",
                    },
                    {
                        "code": "HGB",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "REMISOL",
                        "value": "13.2",
                        "unit": "g/100mL",
                    },
                    {
                        "code": "HCT",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "38.2",
                        "unit": "%",
                    },
                    {
                        "code": "MCV",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "99.4",
                        "unit": "fL",
                    },
                    {
                        "code": "MCH",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "34.4",
                        "unit": "pg/dL",
                    },
                    {
                        "code": "MCHC",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "34.6",
                        "unit": "g/dL",
                    },
                    {
                        "code": "RDW",
                        "comments": [],
                        "date": date,
                        "flags": ["N", "MODERATED ANISOCYTOSE"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "17.0",
                        "unit": "",
                    },
                    {
                        "code": "PLT",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "141.0",
                        "unit": "10^3/mm3",
                    },
                    {
                        "code": "MPV",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "8.6",
                        "unit": "fL",
                    },
                    {
                        "code": "PCT",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "0.1210",
                        "unit": "",
                    },
                    {
                        "code": "PDW",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "15.50",
                        "unit": "",
                    },
                    {
                        "code": "RET#",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "0.0",
                        "unit": "10^3/mm3",
                    },
                    {
                        "code": "RET%",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "1.26",
                        "unit": "%",
                    },
                    {
                        "code": "NE#",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "9.03",
                        "unit": "10^3/mm3",
                    },
                    {
                        "code": "LY#",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "7.5",
                        "unit": "10^3/mm3",
                    },
                    {
                        "code": "MO#",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "0.82",
                        "unit": "10^3/mm3",
                    },
                    {
                        "code": "BA#",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "0.03",
                        "unit": "10^3/mm3",
                    },
                    {
                        "code": "EO#",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "0.02",
                        "unit": "10^3/mm3",
                    },
                    {
                        "code": "NE%",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "51.9",
                        "unit": "%",
                    },
                    {
                        "code": "LY%",
                        "comments": [],
                        "date": date,
                        "flags": ["N", "Lymphocytosis #", "Ly Blasts"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "43.1",
                        "unit": "%",
                    },
                    {
                        "code": "MO%",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "4.7",
                        "unit": "%",
                    },
                    {
                        "code": "BA%",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "0.2",
                        "unit": "%",
                    },
                    {
                        "code": "EO%",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "0.1",
                        "unit": "%",
                    },
                    {
                        "code": "IMR",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "0.23",
                        "unit": "",
                    },
                    {
                        "code": "VMR",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "115.5",
                        "unit": "",
                    },
                    {
                        "code": "VMCS",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "92.6",
                        "unit": "",
                    },
                    {
                        "code": "RFD%",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "0.3",
                        "unit": "%",
                    },
                    {
                        "code": "RFD#",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "0.0",
                        "unit": "",
                    },
                    {
                        "code": "COM1",
                        "date": message_date,
                        "value": "[1] [FROTTIS/] [3] [ANTERIORITE? FROTTIS/]",
                    },
                    {
                        "code": "COM2",
                        "date": message_date,
                        "value": "ALARM 1 VERIFY DIFF",
                    },
                ],
                "sample_id": "0453",
                "rack": "7_18",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_electrophoresys_results(self):
    worklist = self.decode(
        """
    H|\\^&|||REMISOL|||||UPLOAD_SIGN=0\\INTERPRETATION_FLAG=0\\INSTRUMENT_FLAG=0||P|1|20040227143437
    P|1||260833OT106||FLORAX-OT M^||19330826|U
    O|1|23030111^35^3|||R||20031224081000||||||||S
    R|1|^^^SPE^^^^^0|Done|||A||||SYSADM||20040227142832|0
    R|2|^^^TPCZE^^^^^0|101.5|g/L||N||||SYSADM||20040227142832|0
    R|3|^^^Albumin^^^^^0|46.5|g/L||N||||SYSADM||20040227142832|0
    R|4|^^^Alpha1^^^^^0|6.0|g/L||N||||SYSADM||20040227142832|0
    R|5|^^^Alpha2^^^^^0|9.6|g/L||N||||SYSADM||20040227142832|0
    R|6|^^^Beta^^^^^0|7.0|g/L||N||||SYSADM||20040227142832|0
    R|7|^^^Albumin%^^^^^0|45.8|%||N||||SYSADM||20040227142832|0
    R|8|^^^Alpha1%^^^^^0|5.9|%||N||||SYSADM||20040227142832|0
    R|9|^^^Alpha2%^^^^^0|9.5|%||N||||SYSADM||20040227142832|0
    R|10|^^^Beta%^^^^^0|6.9|%||N||||SYSADM||20040227142832|0
    R|11|^^^AG_Ratio^^^^^0|0.9|||N||||SYSADM||20040227142832|0
    R|12|^^^D|11831^0\\11831^15883\\16744^0\\16744^15883\\19791^0\\19791^15883\\23013^0\\23013^15883\\25223^0\\25223^15883\\25776^0\\25776^15883\\27105^0\\27105^15883\\28871^0\\28871^15883|XYCOORD||||
    L|1
    """
    )
    date = datetime.datetime(2004, 2, 27, 14, 28, 32)
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "SPE",
                        "comments": [],
                        "date": date,
                        "flags": ["A"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "Done",
                        "unit": "",
                    },
                    {
                        "code": "TPCZE",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "101.5",
                        "unit": "g/L",
                    },
                    {
                        "code": "Albumin",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "46.5",
                        "unit": "g/L",
                    },
                    {
                        "code": "Alpha1",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "6.0",
                        "unit": "g/L",
                    },
                    {
                        "code": "Alpha2",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "9.6",
                        "unit": "g/L",
                    },
                    {
                        "code": "Beta",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "7.0",
                        "unit": "g/L",
                    },
                    {
                        "code": "Albumin%",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "45.8",
                        "unit": "%",
                    },
                    {
                        "code": "Alpha1%",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "5.9",
                        "unit": "%",
                    },
                    {
                        "code": "Alpha2%",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "9.5",
                        "unit": "%",
                    },
                    {
                        "code": "Beta%",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "6.9",
                        "unit": "%",
                    },
                    {
                        "code": "AG_Ratio",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "0.9",
                        "unit": "",
                    },
                    {
                        "code": "D",
                        "graph": {
                            "points": [
                                "11831",
                                "0",
                                "11831",
                                "15883",
                                "16744",
                                "0",
                                "16744",
                                "15883",
                                "19791",
                                "0",
                                "19791",
                                "15883",
                                "23013",
                                "0",
                                "23013",
                                "15883",
                                "25223",
                                "0",
                                "25223",
                                "15883",
                                "25776",
                                "0",
                                "25776",
                                "15883",
                                "27105",
                                "0",
                                "27105",
                                "15883",
                                "28871",
                                "0",
                                "28871",
                                "15883",
                            ]
                        },
                        "value": "GRAPHE",
                    },
                ],
                "sample_id": "23030111",
                "rack": "3_35",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_control(self):
    worklist = self.decode(
        """
    H|\\^&|||REMISOL|||||UPLOAD_SIGN=1\\INTERPRETATION_FLAG=0\\INSTRUMENT_FLAG=1||E1394-97|20040309112633
    P|1||0000000||Unknown^|||U
    O|1|12345^1^1^1234^^Name|||R||20040309105932||||Q||||S
    R|1|^^^NA^^^^^0|1|mmol/L||N||||SYSADM||20040309110129|0
    R|2|^^^K^^^^^0|2.0|mmol/L||N||||SYSADM||20040309110129|0
    R|3|^^^CL^^^^^0|3.0|meq/L||N||||SYSADM||20040309110129|0
    R|4|^^^CO2^^^^^0|4|meq/L||N||||SYSADM||20040309110129|0
    L|1
    """
    )
    date = datetime.datetime(2004, 3, 9, 11, 1, 29)
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "NA",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "1",
                        "unit": "mmol/L",
                    },
                    {
                        "code": "K",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "2.0",
                        "unit": "mmol/L",
                    },
                    {
                        "code": "CL",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "3.0",
                        "unit": "meq/L",
                    },
                    {
                        "code": "CO2",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "0",
                        "operator": "SYSADM",
                        "value": "4",
                        "unit": "meq/L",
                    },
                ],
                "sample_id": "12345",
                "control_name": "12345",
                "control_lot": "12345",
                "type": "control",
            }
        ],
    }
    assert worklist == expected


def test_control_2(self):
    worklist = self.decode(
        """
    H|\\^&|||REMISOL||||||UPLOAD_SIGN=1\\INTERPRETATION_FLAG=1\\INSTRUMENT_FLAG=0|P|1|20220112143347
    P|1||0000000||Generique^|||U
    O|1|CQ-MICR-2|||R||||||Q||||U
    R|1|^^^MAL010^^^^^0|59.0000|mg/L||N||||BIO-R2KSRV2||20220112143336|AU2
    L|1

    """
    )
    date = datetime.datetime(2022, 1, 12, 14, 33, 36)
    expected = {
        "context": "result",
        "samples": [
            {
                "control_lot": "CQ-MICR-2",
                "control_name": "CQ-MICR-2",
                "results": [
                    {
                        "code": "MAL010",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "AU2",
                        "operator": "BIO-R2KSRV2",
                        "unit": "mg/L",
                        "value": "59.0000",
                    }
                ],
                "sample_id": "CQ-MICR-2",
                "type": "control",
            }
        ],
    }
    assert worklist == expected


def test_query(self):
    result = self.decode(
        """
    H|\\^&|||REMISOL|||||||P|1|20010530155109
    Q|1|^342244||||||||||O
    L|1"""
    )
    expected = {
        "context": "query",
        "samples": [{"sample_id": "342244", "type": "patient"}],
    }
    assert result == expected


def test_tube_arrival(self):
    worklist = self.decode(
        """
    H|\\^&
    M|1|101|SAMPLE123|20020313151515|1
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "SAMPLE123",
                "results": [
                    {
                        "code": "TUBE ARRIVAL",
                        "date": datetime.datetime(2002, 3, 13, 15, 15, 15),
                        "value": "1",
                    }
                ],
            }
        ],
    }
    assert worklist == expected


def test_tube_tracking(self):
    worklist = self.decode(
        """
    H|\\^&
    M|1|103|Sample123^3000^500|Sample123.1^1000|1|2|20020815120000|1|2|1|10|23|45|115^LX1|2
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "Sample123",
                "rack": "115",
                "results": [
                    {
                        "code": "TUBE TRACKING",
                        "date": datetime.datetime(2002, 8, 15, 12),
                        "value": "2",
                    },
                    {"code": "Initial Vol", "value": "3000"},
                    {"code": "Remain Vol", "value": "500"},
                    {"code": "Aliquot Tube ID", "value": "Sample123.1"},
                    {"code": "Requested Vol", "value": "1000"},
                    {"code": "Tube status", "value": "1"},
                    {"code": "Tube error", "value": "2"},
                    {"code": "Track Line ID", "value": "1"},
                    {"code": "Unit", "value": "1"},
                    {"code": "Position", "value": "10"},
                    {"code": "Location", "value": "23"},
                    {"code": "Shelf", "value": "45"},
                    {"code": "Workstation name", "value": "LX1"},
                    {"code": "Tray status", "value": "2"},
                ],
            }
        ],
    }
    assert worklist == expected


def test_tracking_dxa(self):
    worklist = self.decode(
        """
    H|\\^&
    M|1|103|Sample123^3000^1500^500^1000|Sample123.1^1000^400^200^300|20141212110042|OUTPUT^1|STORAGE^2|Storage|4|88AE48EB-D1D2-494C-B780-B0821AB29E91|A6||TrackToOutput
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "88AE48EB-D1D2-494C-B780-B0821AB29E91",
                "results": [
                    {
                        "code": "TUBE TRACKING",
                        "date": datetime.datetime(2014, 12, 12, 11, 0, 42),
                        "value": "Storage",
                    },
                    {"code": "Specimen Total Volume", "value": "3000"},
                    {"code": "Specimen Cruor Volume", "value": "1500"},
                    {"code": "Specimen Gel Volume", "value": "500"},
                    {"code": "Specimen Serum Volume", "value": "1000"},
                    {"code": "Aliquot ID", "value": "Sample123.1"},
                    {"code": "Aliquot Total Volume", "value": "1000"},
                    {"code": "Aliquot Cruor Volume", "value": "400"},
                    {"code": "Aliquot Gel Volume", "value": "200"},
                    {"code": "Aliquot Serum Volume", "value": "300"},
                    {"code": "Frame Type", "value": "OUTPUT"},
                    {"code": "Frame Index", "value": "1"},
                    {"code": "Subsystem Type", "value": "STORAGE"},
                    {"code": "Subsystem Index", "value": "2"},
                    {"code": "Drawer ID", "value": "4"},
                    {"code": "Position in Rack", "value": "A6"},
                    {"code": "Sample Tracking Trigger", "value": "TrackToOutput"},
                ],
                "sample_id": "Sample123",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected
